/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.Objects;

/**
 *
 * @author johan
 */
public abstract class AbstractEntity implements Entity, Serializable {

    private AgroURI uri;
    private transient PropertyChangeSupport propertyChangeSupport;

    // FIXME: remove once all implementing entities can use the URI constructor
    @Deprecated
    public AbstractEntity() {
    }

    public AbstractEntity(AgroURI uri) {
        this.uri = uri;
    }

    public AbstractEntity(EnterpriseIdProvider eip, ItemIdType itemIdType) {
        this(new AgroURIFactory(eip).createURI(itemIdType));
    }

    @Override
    public String getItemIdNumber() {
        return uri.getItemIdNumber();
    }

    @Override
    public ItemIdType getItemIdType() {
        return uri.getItemIdType();
    }

    @Override
    public AgroURI getURI() {
        return uri;
    }

    /**
     * 
     * @param uri
     * @deprecated URI should be immutable.
     */
    @Deprecated
    protected void setURI(AgroURI uri) {
        this.uri = uri;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 13 * hash + Objects.hashCode(this.uri);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final AbstractEntity other = (AbstractEntity) obj;

        return Objects.equals(this.getURI(), other.getURI());
    }

    protected PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    @Override
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        assert listener != null;
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }

    @Override
    public void addPropertyChangeListener(String property, PropertyChangeListener listener) {
        assert listener != null;
        getPropertyChangeSupport().addPropertyChangeListener(property, listener);
    }

    @Override
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }

    @Override
    public void removePropertyChangeListener(String property, PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(property, listener);
    }
}
